#include "set.h"

#include "asndata.def"

void main()
{
	ofile os( "certset.ber" ) ;

	Certificate c ;

	c.cert().version().set( Version::v3 ) ;
	c.cert().serialNumber().set( "'0102030405060708090A0B0C0D0E0F'H" ) ;
	c.cert().signature().set( sha1WithRsaSignature ) ;
	Name nm ;
	nm.set( id_at_countryName, PrintableString( "ES" ) ) ;
	nm.set( id_at_organization, PrintableString( "Unique ID assigned by Brand" ) ) ;
	nm.set( id_at_commonName, PrintableString( "BrandID" ) ) ;
	c.cert().issuer().set( nm ) ;
	c.cert().validity().notBefore().set( time(0) ) ;
	c.cert().validity().notAfter().set( Asn1Time::offset( time(0), 1 ) ) ;
	nm.clear() ;
	nm.set( id_at_countryName, PrintableString( "ES" ) ) ;
	nm.set( id_at_organization, PrintableString( "BrandID" ) ) ;
	nm.set( id_at_organizationalUnit, PrintableString( "Name of Acquiring Financial Institution" ) ) ;
	nm.set( id_at_commonName, PrintableString( "Unique Payment Gateway ID" ) ) ;
	c.cert().subject().set( nm ) ;
	c.cert().subjectPublicKeyInfo().algorithm().set( sha1WithRsaSignature ) ;
	c.cert().subjectPublicKeyInfo().subjectPublicKeyBits().set( "'3068026100BEAA8B7754A3AFCA779F2FB0CF4388FFA66D79555B618C68EC481E8A8638A4FE19B862171D9D0F472CFF638F299104D152BC7F67B6B28F7455C133216C8FAB019524C8B273939D226150A935FB9D575032EF56525093ABB18894785615C61C8B0203010001'H" ) ;
	c.signatureAlgorithm().set( sha1WithRsaSignature ) ;
	c.signature().set( "'5C01BDB54188877A0ED30E6B3ABF04EA04CB5F61723CA3BD78F56617FE373AABEB67BFB7DAA838F6331571752FB98C91A0E487BA4B43A0228FD3A9864389E6505C01BDB54188877A0ED30E6B3ABF04EA04CB5F61723CA3BD78F56617FE373AABEB67BFB7DAA838F6331571752FB98C91A0E487BA4B43A0228FD3A9864389E650'H" ) ;

	// extensions

	AuthorityKeyIdentifier ak ;
	nm.clear() ;
	nm.set( id_at_countryName, PrintableString( "ES" ) ) ;
	nm.set( id_at_organization, PrintableString( "Name of organization responsible for the Root Certificate" ) ) ;
	ak.set( nm, 13 ) ;
	c.cert().extensions(0).set( id_ce_authorityKeyIdentifier, ak ) ;

	c.cert().extensions(1).set( id_ce_keyUsage, KeyUsage( KeyUsage::keyEncipherment ), TRUE ) ;

	PrivateKeyUsagePeriod pku ;
	pku.notBefore().set( Asn1Time::offset( time(0), -1 ) ) ;
	pku.notAfter().set( Asn1Time::offset( time(0), 1 ) ) ;
	c.cert().extensions(2).set( id_ce_privateKeyUsagePeriod, pku, TRUE ) ;

	CertificatePoliciesSyntax pols ; // mere example id
	pols.policyInformation(0).policyIdentifier().set( ObjectIdentifier( 1, 2, 65536 ) ) ;
	c.cert().extensions(3).set( id_ce_certificatePolicies, pols ) ;

	GeneralNames gn ;
	gn.generalName(0).rfc822Name().set( "pgwy@acquirer.brand.es" ) ;
	gn.generalName(1).uniformResourceIdentifier().set( "http://acquirer.brand.es/pgwy/" ) ;
	c.cert().extensions(4).set( id_ce_subjectAltName, gn ) ;

	c.cert().extensions(5).set( id_ce_basicConstraints, BasicConstraintsSyntax(), TRUE ) ;

	gn.generalName(0).rfc822Name().set( "bca@brand.es" ) ;
	gn.generalName(1).uniformResourceIdentifier().set( "http://brand.es/bca/" ) ;
	c.cert().extensions(6).set( id_ce_issuerAltName, gn ) ;

	CertificateTypeSyntax ct ;
	ct.set( CertificateTypeSyntax::pgwy ) ;
	c.cert().extensions(7).set( id_set_certificateType, ct, TRUE ) ;

	c.cert().extensions(8).set( id_set_cardCertRequired, Boolean(TRUE) ) ;

	TunnelingSyntax tn ;
	tn.tunneling().set( TRUE ) ;
	tn.tunnelAlgIDs(0).set( des_cbc ) ;
	tn.tunnelAlgIDs(1).set( des_cdmf ) ;
	c.cert().extensions(9).set( id_set_tunneling, tn ) ;

	SETQualifierSyntax sq ;
	sq.policyDigest().set( "'453E6F23658787AB653EB33453D270093456A367'H" ) ;
	sq.digestAlgorithm().set( sha1 ) ;
	sq.terseStatement().set( "A statement declaring any disclaimers associated with the issuing of the certificate." ) ;
	sq.policyURL().set( "http://brand.es/policies/thisPolicy" ) ;
	sq.policyEmail().set( "thisPolicy@policies.brand.es" ) ;
	c.cert().extensions(10).set( id_set_setQualifier, sq, TRUE ) ;

	c.encode( os ) ;
}
